
// ::::::::::::::::::::::::
// magnitude response plot

(
var coefs, calcCoefs, calcMagnitudeResponse, calcMagnitudes, magnitudes, sampleDur, maxdb, minFreq, maxFreq, calcHeight, filter, filterKind, freq, q, gain, updateFilter, knobsSize;
var cv, uv, window, freqMarkers, levelMarkers;

sampleDur = 44100.0.reciprocal;
~maxdb=40.0;
freqMarkers = [ (freq:10, tag:"10"), (freq:20), (freq:30), (freq:40), (freq:50), (freq:60), (freq:70), (freq:80), (freq:90), (freq:100, tag:"100"), (freq:200), (freq:300), (freq:400), (freq:500), (freq:600), (freq:700), (freq:800), (freq:900), (freq:1000, tag:"1k"), (freq:2000), (freq:3000), (freq:4000), (freq:5000, tag:"5k"), (freq:6000), (freq:7000), (freq:8000), (freq:9000), (freq:10e3, tag:"10k"), (freq:11e3), (freq:12e3), (freq:13e3), (freq:14e3), (freq:15e3), (freq:16e3), (freq:17e3), (freq:18e3), (freq:19e3), (freq:20e3) ];

levelMarkers = [3, 6, 12, 18, 24, 30, 36, 42, 48, 52, 58];

calcCoefs = #{ arg freq, rq, gain, sampleDur;
	var alpha, a, w0, w0cos, a0, a1, a2, b0, b1, b2;
    a = pow(10.0, gain/40.0);
    w0 = 2pi * freq * (sampleDur?44100.reciprocal);
    alpha = w0.sin / (2 * rq.reciprocal);
	w0cos = w0.cos;

	b0 =  1.0 + (alpha * a);
	b1 = -2.0 * w0cos;
	b2 =  1.0 - (alpha * a);
	a0 =  1.0 + (alpha / a);
	a1 = -2.0 * w0cos;
	a2 =  1.0 - (alpha / a);

	b0 = b0 / a0;
	b1 = b1 / a0;
	b2 = b2 / a0;
	a1 = a1 / a0;
	a2 = a2 / a0;
	a0 = 1;
	[b0, b1, b2, a1.neg, a2.neg];
};

calcMagnitudeResponse = #{ arg freq, sampleDur, b0, b1, b2, a1, a2;
    var w, numerator, denominator, magnitude;
	w = 2pi * freq * sampleDur;
	a1 = a1.neg;
	a2 = a2.neg;
	numerator = (b0*b0)+(b1*b1)+(b2*b2)+(2*((b0*b1)+(b1*b2))*w.cos)
	  +(2*b0*b2*(2.0*w).cos);
	denominator = 1.0+(a1*a1)+(a2*a2)+(2*(a1+(a1*a2))*w.cos)
	  +(2*a2*(2.0*w).cos);
	magnitude = (numerator / denominator).pow(1/2).ampdb;
};

minFreq = 10.0;
maxFreq = 20e3;
~maxdb=maxdb=12.0;
freq=1000.0; q=1.0; gain=6.0;
filter = BPeakEQ;
~coefs = filter.coefs(freq, q.reciprocal, gain, sampleDur);
//~coefs = calcCoefs.(freq, q.reciprocal, gain, sampleDur);

coefs = ~coefs;

window = Window.new("magnitude response", Rect(10,200,400,260));
window.view.minSize = Point(400,260);
window.view.decorator = FlowLayout(window.view.bounds);
//window.alwaysOnTop=true;
~window = window;

uv = UserView.new(window, Point(400,150)-8);
uv.resize_(5);
uv.background_(Color.blue(0.1,0.1));
~view = uv;

calcMagnitudes = {
	var zeroY, viewHeight, viewWidth;
	viewWidth = uv.bounds.width;
	viewHeight = uv.bounds.height;
	zeroY = (viewHeight * 0.5);

	magnitudes = Array.newClear(viewWidth);

	viewWidth.do { arg x;
		var freq, magnitude;
		freq = x.linexp(0, viewWidth, minFreq, maxFreq);
		magnitude = calcMagnitudeResponse.(freq, sampleDur, *coefs);
		magnitudes[x] = magnitude.linlin(maxdb.neg,maxdb,viewHeight,0);
	};
};

updateFilter = {
	if (filter == BPeakEQ or:{ filter == BLowShelf } or:{ filter == BHiShelf }) {
		~coefs = filter.coefs(freq, q.reciprocal, gain, sampleDur);
	}{
		~coefs = filter.coefs(freq, q.reciprocal, sampleDur);
	};
	magnitudes=nil; uv.refresh;
};

uv.drawFunc_({ arg view;
	var zeroY, maxX, viewWidth, viewHeight;
	viewWidth = maxX = view.bounds.width;
	viewHeight = view.bounds.height;
	zeroY = (viewHeight * 0.5);

	Pen.font = Font("Arial", 24);
	//Pen.stringAtPoint("Hello, World!", Point(10,10));

    Pen.strokeColor = Color.black;

    Pen.color = Color.black;
    Pen.width = 0.5;
	Pen.line(Point(0,zeroY), Point(viewWidth,zeroY));

	coefs = ~coefs;
	maxdb = ~maxdb;

	freqMarkers.do { arg marker;
		var x, freq, tag;
		freq = marker.freq;
		x = freq.explin(minFreq, maxFreq, 0, maxX);
		Pen.line(Point(x,0), Point(x,viewHeight));
		tag = marker.tag;
		if (tag.notNil) {
			Pen.stringAtPoint(tag, Point(x+2,4), Font("Arial", 11));
		};
	};

	levelMarkers.do {|marker|
		var y;
		if (marker<maxdb) {
			y  = marker.linlin(maxdb.neg, maxdb, 0, viewHeight);
			Pen.line(Point(0,y), Point(maxX,y));
			y  = marker.neg.linlin(maxdb.neg, maxdb, 0, viewHeight);
			Pen.line(Point(0,y), Point(maxX,y));
		};
	};
	Pen.stroke;

    Pen.color = Color.blue;
	Pen.moveTo(Point(0,zeroY));

	if (magnitudes.size != viewWidth or:{ viewHeight != calcHeight }) {
		calcMagnitudes.value;
		calcHeight = viewHeight;
	};
	viewWidth.do { arg x; Pen.lineTo(Point(x, magnitudes.at(x))); };

	Pen.stroke;
});

knobsSize = Point(60,100);
cv = CompositeView(window,Point(window.view.bounds.width-4,knobsSize.y+8));
cv.resize = 8;
cv.decorator = FlowLayout(cv.bounds);
cv.background = Color.blue(0.1,0.1);

~knobs = [
	EZKnob.new(cv, knobsSize,
		"Freq",
		[10,20e3,'exp'].asSpec,
		{ arg knob;
			freq = knob.value;
			updateFilter.value;
		}, freq),

	EZKnob.new(cv, knobsSize,
		"Q",
		[0.01,100,'exp'].asSpec,
		{ arg knob;
			q = knob.value;
			updateFilter.value;
		}, q),

	EZKnob.new(cv, knobsSize,
		"Gain",
		[-20,20].asSpec,
		{ arg knob;
			gain = knob.value;
			updateFilter.value;
		}, gain)
];

filterKind = [ BPeakEQ, BLowShelf, BHiShelf, BLowPass, BHiPass, BBandPass, BBandStop ];
PopUpMenu(cv,Point(100,20))
.items_(["BPeakEQ", "BLowShelf", "BHiShelf", "BLowPass", "BHiPass", "BBandPass", "BBandStop"])
.action_({ arg view;
	filter = filterKind.at(view.value);
	updateFilter.value;
});

PopUpMenu(cv,Point(70,20))
.items_(["3.0", "6.0", "12.0", "20.0", "40.0", "80.0"])
.action_({ arg view;
	~maxdb=maxdb=[3,6,12,20,40,80].at(view.value);
	magnitudes=nil; uv.refresh;
})
.value_(2);

window.front;
~redraw = { magnitudes=nil; uv.refresh; };
)

~maxdb=80.0; ~redraw.value;
~maxdb=40.0; ~redraw.value;
~maxdb=20.0; ~redraw.value;
~maxdb=12.0; ~redraw.value;
~maxdb=6.0; ~redraw.value;
~coefs = BPeakEQ.coefs(1000.0, 1.0.reciprocal, 6.0); ~redraw.value;
~coefs = BLowShelf.coefs(120.0, 4.0.reciprocal, 6.0); ~redraw.value;
~coefs = BHiShelf.coefs(1000.0, 4.0.reciprocal, 6.0); ~redraw.value;
~coefs = BLowPass.coefs(220.0, 0.7.reciprocal); ~redraw.value;
~coefs = BHiPass.coefs(1000.0, 0.7.reciprocal); ~redraw.value;
~coefs = BBandStop.coefs(1000.0, bw:1.0); ~redraw.value;
~coefs = BBandPass.coefs(1000.0, bw:0.5); ~redraw.value;
