//------------------------------------------------------------------
//	DAFX, Digital Audio Effects (Wiley ed.)
//	chapter 2 	: filters
//	section 2.3 : Equalizers
//	page 53 	: second order shelving filter design
//------------------------------------------------------------------

// low-freq shelving boost

(
SynthDef("sos-lowBoost", { arg out, frequency=100.0, gain=0, bypass=0, amp=1.0;
	var in, fx, dry, v0g, fc, denom, a0, a1, a2, b1, b2;
	var fc_squared, tmp1, tmp2;

	v0g = pow(10.0, gain / 20.0);
	fc	= (pi*frequency*SampleDur.ir).tan;
	fc_squared = fc.pow(2);
	tmp1 = (2.sqrt*fc) + fc_squared;
	tmp2 = ((2*v0g).sqrt * fc) + (v0g*fc_squared);
	denom = 1.0 + tmp1;

	a0 = (1 + tmp2) / denom;
	a1 = 2 * (v0g*fc_squared - 1) / denom;
	a2 = (1 - tmp2) / denom;
	b1 = -2 * (fc_squared - 1) / denom;
	b2 = (1 - tmp1).neg / denom;

	dry = In.ar(2,2);
	fx  = SOS.ar(dry, a0, a1, a2, b1, b2, amp);

	Out.ar(out, (fx * (1 - bypass))+(dry * bypass));
}).add;
)

a=Synth("sos-lowBoost", [\frequency, 100.0, \gain, 0, \bypass, 0, \amp, 1.0]);
a.set(\frequency, 480.0, \gain, 3, \bypass, 0, \amp, 1.0);

a.set(\bypass, 1);
a.set(\bypass, 0);


(
SOSKernel.filters.put(\lowBoost, { arg frequency=100.0, gain=0.0, sampleDur=(44100.reciprocal);
	var v0g, fc, denom, fc_pow2, tmp1, tmp2;
	var a0, a1, a2, b1, b2;

	v0g = pow(10.0, gain / 20.0);
	fc	= (pi*frequency*sampleDur).tan;
	fc_pow2 = fc.pow(2);
	tmp1 = (2.sqrt*fc) + fc_pow2;
	tmp2 = ((2*v0g).sqrt * fc) + (v0g*fc_pow2);
	denom = 1.0 + tmp1;

	a0 = (1 + tmp2) / denom;
	a1 = 2 * (v0g*fc_pow2 - 1) / denom;
	a2 = (1 - tmp2) / denom;
	b1 = -2 * (fc_pow2 - 1) / denom;
	b2 = (1 - tmp1).neg / denom;
	
	[a0, a1, a2, b1, b2]
});
)

k=SOSKernel.new;
v=k.view;
k.filterKey
k.filter
k.params
k.coefs

k.params;
k.params.frequency = 400.0;
k.params.gain = 2.7;
k.paramsChanged;
k.changed(\coefs);
v.refresh

k.inspect
k.selectFilter('BLowPass');
k.selectFilter('BLowShelf');
k.selectFilter('BPeakEQ');

v.refresh;
k.paramsChanged

MagnitudeResponseView
LowBoost

k.params
k.freqKey
k.gainKey
k.qKey
k.qIsReciprocal

k.frequency
k.gain
k.q

SOSKernel.filters.put('FOSlowpass', FOSlowpass);
SOSKernel.filters.put('FOShighpass', FOShighpass);

FOSlowpass