// BEQSuite-plus - blackrain 2018
// Robert Bristow-Johnson biquad Eqs. yes. again,
// this time the lang-side implementation of the BEQSuite *sc method
// *coefs doesnt make use of the SampleDur.ir UGen, so pure lang side use of these methods is ok.
//
// NOTE:
//   *coefs arguments ARE different from *sc method.
//   ALSO: while the *sc method relays on the SampleRate and SampleDur UGens (that's server side), *coefs methods
//   are a pure lang side implementation.

+ BLowPass {
	*coefs { arg freq=100.0, rq=1.0, sampleDur=(44100.0.reciprocal);
		var w0, cos_w0, tmp, alpha, a0, a1, a2, b0rz, b1, b2;
		w0 = 2pi * freq * sampleDur;
		cos_w0 = w0.cos;
		alpha = w0.sin * 0.5 * rq;
		tmp = 1 - cos_w0;
		b0rz = (1 + alpha).reciprocal;
		a0 = tmp * 0.5 * b0rz;
		a1 = tmp * b0rz;
		a2 = a0;
		b1 = 2 * cos_w0 * b0rz;
		b2 = 1 - alpha * b0rz.neg;
		^[a0, a1, a2, b1, b2];
	}
}

+ BHiPass {
	*coefs { arg freq=1200.0, rq=1.0, sampleDur=(44100.reciprocal);
		var i, w0, cos_w0, alpha, a0, a1, b0rz, b1, b2;
		w0 =  2pi * freq * sampleDur;
		cos_w0 = w0.cos; i = 1 + cos_w0;
		alpha = w0.sin * 0.5 * rq;
		b0rz = (1 + alpha).reciprocal;
		a0 = i * 0.5 * b0rz;
		a1 = i.neg * b0rz;
		b1 = cos_w0 * 2 * b0rz;
		b2 = (1 - alpha) * b0rz.neg;
		^[a0, a1, a0, b1, b2];
	}
}

+ BAllPass {
	*coefs { arg freq=1000.0, rq=1.0, sampleDur=(44100.reciprocal);
		var w0, alpha, a0, b1, b0rz;
		w0 = 2pi * freq * sampleDur;
		alpha = w0.sin * 0.5 * rq;
		b0rz = (1 + alpha).reciprocal;
		a0 = (1 - alpha) * b0rz;
		b1 = 2.0 * w0.cos * b0rz;
		^[a0, b1.neg, 1.0, b1, a0.neg];
	}
}

+ BBandPass {
	*coefs { arg freq=1000.0, bw=1.0, sampleDur=(44100.reciprocal);
		var w0, sin_w0, alpha, a0, b0rz, b1, b2;
		w0 = 2pi * freq * sampleDur;
		sin_w0 = w0.sin;
		alpha = sin_w0 * sinh(0.34657359027997 * bw * w0 / sin_w0);
		b0rz = (1 + alpha).reciprocal;
		a0 = alpha * b0rz;
		b1 = w0.cos * 2 * b0rz;
		b2 = (1 - alpha) * b0rz.neg;
		^[a0, 0.0, a0.neg, b1, b2];
	}
}

+ BBandStop {
	*coefs { arg freq=1000.0, bw=1.0, sampleDur=(44100.reciprocal);
		var w0, sin_w0, alpha, b1, b2, b0rz;
		w0 = 2pi * freq * sampleDur;
		sin_w0 = w0.sin;
		alpha = sin_w0 * sinh(0.34657359027997 * bw * w0 / sin_w0);
		b0rz = (1 + alpha).reciprocal;
		b1 = 2.0 * w0.cos * b0rz;
		b2 = (1 - alpha) * b0rz.neg;
		^[b0rz, b1.neg, b0rz, b1, b2];
	}
}

+ BPeakEQ {
	*coefs { arg freq=1000.0, rq=1.0, db=0.0, sampleDur=(44100.reciprocal);
		var a, w0, alpha, a0, a2, b1, b2, b0rz;
		a = pow(10, db/40);
		w0 = 2pi * freq * sampleDur;
		alpha = w0.sin * 0.5 * rq;
		b0rz = (1 + (alpha / a)).reciprocal;
		a0 = (1 + (alpha * a)) * b0rz;
		a2 = (1 - (alpha * a)) * b0rz;
		b1 = 2.0 * w0.cos * b0rz;
		b2 = (1 - (alpha / a)) * b0rz.neg;
		^[a0, b1.neg, a2, b1, b2];
	}
}

+ BLowShelf {
	*coefs { arg freq=100.0, rs=1.0, db=0.0, sampleDur=(44100.reciprocal);
		var a, w0, sin_w0, cos_w0, alpha, i, j, k, a0, a1, a2, b0rz, b1, b2;
		a = pow(10, db/40);
		w0 = 2pi * freq * sampleDur;
		cos_w0 = w0.cos;
		sin_w0 = w0.sin;
		alpha = sin_w0 * 0.5 * sqrt((a + a.reciprocal) * (rs - 1) + 2.0); // S
		i = (a+1) * cos_w0;
		j = (a-1) * cos_w0;
		k = 2 * sqrt(a) * alpha;
		b0rz = ((a+1) + j + k).reciprocal;
		a0 = a * ((a+1) - j + k) * b0rz;
		a1 = 2 * a * ((a-1) - i) * b0rz;
		a2 = a * ((a+1) - j - k) * b0rz;
		b1 = 2.0 * ((a-1) + i) * b0rz;
		b2 = ((a+1) + j - k) * b0rz.neg;
		^[a0, a1, a2, b1, b2];
	}
}

+ BHiShelf {
	*coefs { arg freq=4200.0, rs=1.0, db=0.0, sampleDur=(44100.reciprocal);
		var a, w0, sin_w0, cos_w0, alpha, i, j, k, a0, a1, a2, b0rz, b1, b2;
		a = pow(10, db/40);
		w0 = 2pi * freq * sampleDur;
		cos_w0 = w0.cos;
		sin_w0 = w0.sin;
		alpha = sin_w0 * 0.5 * sqrt((a + a.reciprocal) * (rs - 1) + 2.0); // S
		i = (a+1) * cos_w0;
		j = (a-1) * cos_w0;
		k = 2 * sqrt(a) * alpha;
		b0rz = ((a+1) - j + k).reciprocal;
		a0 = a * ((a+1) + j + k) * b0rz;
		a1 = -2.0 * a * ((a-1) + i) * b0rz;
		a2 = a * ((a+1) + j - k) * b0rz;
		b1 = -2.0 * ((a-1) - i) * b0rz;
		b2 = ((a+1) - j - k) * b0rz.neg;
		^[a0, a1, a2, b1, b2];
	}
}
