// blackrain

FOSKernel : FilterKernelBase {
	classvar <filters;
	var <a0, <a1, <b1;

	*initClass {
		StartUp.add {
			filters = Dictionary [
				'FOSlowpass' -> KernelFilter.new('FOSlowpass', FOSlowpass, 'cutoff', nil, nil, false),
				'FOShighpass' -> KernelFilter.new('FOShighpass', FOShighpass, 'cutoff', nil, nil, false),
			];
		};
	}

	*new { arg freq=100.0, q=1.0, dBGain=0.0, filterKey='FOSlowpass', sampleRate=44100.0;
		^super.new(freq, q, dBGain, filterKey, sampleRate)
	}
	clearCoefs {
		a0=nil; a1=nil; b1=nil;
	}
    paramsChanged {
		var values = argNames.collect({ arg argName; params.at(argName); });
		#a0, a1, b1 = filter.perform(this.coefsMethod, *values);
        this.changed(\coefs);
    }
	coefs {
		^[ a0, a1, b1 ]
	}
	setCoefs { arg in_a0, in_a1, in_b1;
        a0 = in_a0;
        a1 = in_a1;
        b1 = in_b1;
        this.changed(\coefs);
	}
	magnitudeResponse { arg viewWidth, minFreq=20.0, maxFreq;
		var response, nyq;
		nyq = sampleDur.reciprocal * 0.5;
		maxFreq = maxFreq ?? { nyq };
		maxFreq = maxFreq.min(nyq);

		response = Array.fill(viewWidth, { arg x;
			var freq, cos_w0, numerator, denominator;
			freq = x.linexp(0, viewWidth - 1, minFreq, maxFreq);
			cos_w0 = (2pi*freq*sampleDur).cos;
			numerator   = (a0*a0)+(a1*a1)+(2*a0*a1*cos_w0);
			denominator = 1+(b1*b1)-(2*b1*cos_w0);
			(numerator/denominator).pow(1/2).ampdb;
		});

		^response
	}
	/*
	phaseResponse { arg freq;
		var w, sin_w, cos_w, numerator, denominator;
		w=2pi*freq*sampleDur;
		sin_w=w.sin;
		cos_w=w.cos;
		numerator=(a1+(b1*a0))*sin_w.neg;
		denominator=a0-(b1*a1)+((a1-(b1*a0))*cos_w);
		^atan(numerator/denominator).ampdb;
	}
	*/
	phaseResponse { arg viewWidth, minFreq=20.0, maxFreq;
		var response, nyq;
		nyq = sampleDur.reciprocal * 0.5;
		maxFreq = maxFreq ?? { nyq };
		maxFreq = maxFreq.min(nyq);
		//#a0, a1, b1 = coefs;

		response = Array.fill(viewWidth, { arg x;
			var freq, w, sin_w, cos_w, numerator, denominator;
			freq = x.linexp(0, viewWidth - 1, minFreq, maxFreq);
			w=2pi*freq*sampleDur;
			sin_w=w.sin;
			cos_w=w.cos;
			numerator=(a1+(b1*a0))*sin_w.neg;
			denominator=a0-(b1*a1)+((a1-(b1*a0))*cos_w);
			atan(numerator/denominator).ampdb
		});

		^response
	}
}
