// blackrain

FilterKernelBase {
	var <filterKey, <coefsMethod, <sampleDur, <filter, <enabled=true;
	var <params, <argNames, <freqKey, <gainKey, <qKey,
	<qIsReciprocal=false;

	*new { arg freq, q, dbGain, filterKey, sampleRate=44100.0;
		^super.new.initFKB(freq, q, dbGain, filterKey, sampleRate)
	}
	initFKB { arg inFreq, inQ, indBGain, inFilterKey, inSampleRate;
		freqKey=\freq; gainKey=\db; qKey=\q;
		params = ( freq: inFreq, db: indBGain, q: inQ);
		sampleDur = inSampleRate.reciprocal;
		this.selectFilterType(inFilterKey);
	}
    paramsChanged {
		^this.subclassResponsibility(thisMethod)
    }
	coefs {
		^this.subclassResponsibility(thisMethod)
	}
	setCoefs { arg ...coefs;
		^this.subclassResponsibility(thisMethod)
	}
	magnitudeResponse { arg viewWidth, minFreq, maxFreq;
		^this.subclassResponsibility(thisMethod)
    }
	phaseResponse { arg freq;
		^this.subclassResponsibility(thisMethod)
	}
	clearCoefs {
		^this.subclassResponsibility(thisMethod)
	}
	selectFilterClass { arg inFilterClass;
		this.filterClass_(inFilterClass)
	}
	filterClass_ { arg inFilterClass;
		this.selectFilterType(inFilterClass.asSymbol)
	}
	filterClass {
		^this.filterKey.asClass
	}
	selectFilterType { arg inFilterKey;
		var m, setParamKeys, oldParams, oldFreqKey, oldGainKey, oldQKey,
		restoreFilterValues, kernelFilter ;

		if (filterKey!=inFilterKey) {
			filterKey=inFilterKey;

			this.clearCoefs;

			oldParams=params;
			oldFreqKey=freqKey;
			oldGainKey=gainKey;
			oldQKey=qKey;

			setParamKeys = {
				kernelFilter = this.class.filters.at(filterKey);
				freqKey=kernelFilter.freqKey;
				gainKey=kernelFilter.gainKey;
				qKey=kernelFilter.qKey;
				qIsReciprocal=kernelFilter.isQreciprocal;

				//[ kernelFilter, freqKey, gainKey, qKey, qIsReciprocal ].postln;

				/*
				freqKey = case
				  {argNames.includes(\freq)}      { \freq }
				  {argNames.includes(\frequency)} { \frequency }
				  {argNames.includes(\cutoff)}    { \cutoff };

				gainKey = case
				  {argNames.includes(\db)}   { \db }
				  {argNames.includes(\gain)} { \gain };

				case
				  {argNames.includes(\q)}  { qKey=\q;  qIsReciprocal=false; }
				  {argNames.includes(\rq)} { qKey=\rq; qIsReciprocal=true;  }
				  {argNames.includes(\s)}  { qKey=\s;  qIsReciprocal=false; }
				  {argNames.includes(\rs)} { qKey=\rs; qIsReciprocal=true;  }
				  {argNames.includes(\bw)} { qKey=\bw; qIsReciprocal=false; };
				*/
			};

			restoreFilterValues = {
				if (oldParams.notNil) {

					if (oldFreqKey.notNil and:{freqKey.notNil}) {
						params[freqKey]=oldParams.at(oldFreqKey);
					};

					if (oldGainKey.notNil and:{gainKey.notNil}) {
						params[gainKey]=oldParams.at(oldGainKey);
					};

					if (oldQKey.notNil and:{qKey.notNil}) {
						params[qKey]=oldParams.at(oldQKey);
					};
				};
			};

			filter=this.class.filters.at(filterKey);

			if (filter.notNil) {
				filter = filter.className;



				if (filter.respondsTo(\coefs)) {
					m=filter.class.findMethod(\coefs);
					params=m.makeEnvirFromArgs;
					argNames=m.argNames.drop(1);
					coefsMethod=\coefs;
					setParamKeys.value;
					restoreFilterValues.value;
					this.paramsChanged;
				}{
					if (filter.isKindOf(Function)) {
						params=filter.def.makeEnvirFromArgs;
						argNames=filter.argNames;
						coefsMethod=\value;
						setParamKeys.value;
						restoreFilterValues.value;
						this.paramsChanged;
					};
				};

			};
		};
	}
	setSampleRate { arg inSampleRate;
		if (sampleDur.reciprocal != inSampleRate) {
			sampleDur = inSampleRate.reciprocal;
			this.paramsChanged;
			this.changed(\sampleRate);
		}
	}
	sampleRate_ { arg inSampleRate;
		if (sampleDur.reciprocal != inSampleRate) {
			sampleDur = inSampleRate.reciprocal;
		}
	}
	sampleRate {
		^sampleDur.reciprocal
	}
	setSampleDur { arg inSampleDur;
		if (sampleDur != inSampleDur) {
			sampleDur = inSampleDur;
			this.paramsChanged;
			this.changed(\sampleRate);
		};
	}
	frequency {
		var freq;
		if (freqKey.notNil) {
			freq=params.at(freqKey)
		};
		^freq
	}
	gain {
		var g;
		if (gainKey.notNil) {
			g=params.at(gainKey)
		};
		^g
	}
	q {
		var q;
		if (qKey.notNil) {
			q=params.at(qKey);
			if (qIsReciprocal) {
				q=q.reciprocal;
			};
		};
		^q
	}
	frequency_ { arg inFrequency;
		if (freqKey.notNil) {
			params[freqKey]=inFrequency;
		}
	}
	gain_ { arg inGain;
		if (gainKey.notNil) {
			params[gainKey]=inGain;
		}
	}
	q_ { arg inQ;
		if (qKey.notNil) {
			if (qIsReciprocal) {
				inQ=inQ.reciprocal;
			};
			params[qKey]=inQ;
		}
	}
	setFrequency { arg inFrequency;
		if (freqKey.notNil) {
			if (params.at(freqKey) != inFrequency) {
				params[freqKey]=inFrequency;
				this.paramsChanged;
				this.changed(\frequency);
			}
		}
	}
	setGain { arg inGain;
		var gain;
		if (gainKey.notNil) {
			gain=params.at(gainKey);
			if (gain.notNil) {
				if (gain != inGain) {
					params[gainKey]=inGain;
					this.paramsChanged;
					this.changed(\gain);
				};
			};
		};
	}
	setQ { arg inQ;
		var q;
		if (qKey.notNil) {
			q=params.at(qKey);
			if (q.notNil) {
				if (q != inQ) {
					if (qIsReciprocal) {
						inQ = inQ.reciprocal;
					};
					params[qKey]=inQ;
					this.paramsChanged;
					this.changed(\q);
				};
			};
		};
	}
	enabled_ { arg flag;
		if (enabled!= flag) {
			enabled = (flag==true);
			this.paramsChanged;
			this.changed(\enabled);
		}
	}
	infoString {
		var infoStr, filterName, qStr;

		filterName = filterKey.asString;
		if (this.filter.isKindOf(Function)) {
			filterName = "Func:" + filterName;
		};

		infoStr = format("%", filterName);

		if (freqKey.notNil) {
			infoStr = infoStr + format("Frequency: % Hz", this.frequency.trunc(0.0001));
		};

		if (qKey.notNil) {
			case
			{qKey==\q}  {qStr="%  Q: %";}
			{qKey==\rq} {qStr="%  Q: %";}
			{qKey==\s}  {qStr="%  S: %";}
			{qKey==\rs} {qStr="%  S: %";}
			{qKey==\bw} {qStr="% BW: %";};
			infoStr = format(qStr, infoStr, this.q.trunc(0.01));
		};

		if (gainKey.notNil) {
			infoStr = infoStr + format("Gain: % dBs", this.gain.trunc(0.01));
		};

		if (this.enabled.not) {
			infoStr = format("Disabled - %", infoStr);
		};

		^infoStr
	}
	snapshot {
		^params.copy.put(\filterKey, filterKey);
	}
	restore { arg state;
        var inFilterKey;
        inFilterKey=state.at(\filterKey);
        if (inFilterKey.notNil) {
            this.selectFilterType(inFilterKey);
            params = ().putAll(state);
            params.removeAt(\filterKey);
            this.paramsChanged;
        };
	}
	view { arg parent, bounds, title;
		^FilterKernelView(this,parent,bounds,title)
	}
}
