// blackrain

OnePoleKernel : FilterKernelBase {
	classvar <filterKind;
	var a0, a1, b1;

	*initClass {
		StartUp.add {
			filterKind = Dictionary [
				//'OnePoleLPF'->OnePoleLPF
				'OnePoleLPF'  -> KernelFilter.new('OnePoleLPF', OnePoleLPF, 'cutoff', nil, nil, false),
			];
		};
	}

	*new { arg frequency=1000.0, filterKey, sampleRate;
		sampleRate = sampleRate ? 44100.0;
		filterKey = filterKey ? 'OnePoleLPF';
		^super.new(frequency, filterKey, sampleRate.reciprocal).paramsChanged
	}
    paramsChanged {
        #a0, a1, b1 = filterKind.at(filterKey).coefs(this.frequency, this.sampleDur);
    }
	coefs {
		^[ a0, a1, b1 ]
	}
	setCoefs { arg in_a0, in_a1, in_b1;
        a0 = in_a0;
        a1 = in_a1;
        b1 = in_b1;
	}
	clearCoefs {
		a0=nil; a1= nil; b1=nil;
	}
	magnitudeResponse { arg viewWidth, minFreq=20.0, maxFreq;
		var magnitudes, nyq;
		nyq = this.sampleDur.reciprocal * 0.5;
		maxFreq = maxFreq ?? { nyq };
		maxFreq = maxFreq.min(nyq);

		magnitudes = Array.fill(viewWidth, { arg x;
			var freq, cos_w, numerator, denominator;

			freq = x.linexp(0, viewWidth, minFreq, maxFreq);

			cos_w = (2pi*freq*sampleDur).cos;
			numerator   = a0*a0;
			denominator = 1+(b1*b1)-(2*b1*cos_w);
			(numerator/denominator).pow(1/2).ampdb
		});

		^magnitudes
	}
	phaseResponse { arg freq;
		var w, response;
		w=2pi*freq*sampleDur;
		response=atan2(b1*w.sin,1+(b1*w.cos));
		if (b1 < 0) {
			^response=response + pi;
		};
		^response
	}
	printOn { arg stream;
		stream
			<< this.class.asCompileString << "(" << this.frequency << ", " << this.filterKey.asCompileString << ", " << this.sampleRate << ")"
	}
}
