// blackrain

SOSKernel : FilterKernelBase {
	classvar <filters;
	var <a0=0, <a1=0, <a2=0, <b1=0, <b2=0;

	*initClass {
		StartUp.add {
			filters=Dictionary [
				'BLowPass'  -> KernelFilter.new('BLowPass', BLowPass, 'freq', nil, 'rq', true),
				'BHiPass'   -> KernelFilter.new('BHiPass', BHiPass, 'freq', nil, 'rq', true),
				'BAllPass'  -> KernelFilter.new('BAllPass', BAllPass, 'freq', nil, nil, false),
				'BBandPass' -> KernelFilter.new('BBandPass', BBandPass, 'freq', 'bw', nil, false),
				'BBandStop' -> KernelFilter.new('BBandStop', BBandStop, 'freq', 'bw', nil, false),
				'BPeakEQ'   -> KernelFilter.new('BPeakEQ', BPeakEQ, 'freq', 'db', 'rq', true),
				'BLowShelf' -> KernelFilter.new('BLowShelf', BLowShelf, 'freq', 'db', 'rs', true),
				'BHiShelf'  -> KernelFilter.new('BHiShelf', BHiShelf, 'freq', 'db', 'rs', true),
                'LowBoost'  -> KernelFilter.new('LowBoost', LowBoost, 'freq', 'db', nil, false),
			];
		};
	}
	*new { arg freq=100.0, q=0.7, dBGain=0.0, filterKey='BPeakEQ', sampleRate=44100.0;
		^super.new(freq, q, dBGain, filterKey, sampleRate);
	}
	clearCoefs {
		//a0=nil; a1=nil; a2=nil; b1=nil; b2=nil;
		a0=0; a1=0; a2=0; b1=0; b2=0;
	}
	paramsChanged {
		var values = argNames.collect({ arg argName; params.at(argName); });
		#a0, a1, a2, b1, b2 = filter.perform(this.coefsMethod, *values);
        this.changed(\coefs);
	}
	coefs {
		^[ a0, a1, a2, b1, b2 ]
	}
	setCoefs { arg in_a0, in_a1, in_a2, in_b1, in_b2;
        a0 = in_a0;
        a1 = in_a1;
        a2 = in_a2;
        b1 = in_b1;
        b2 = in_b2;
        this.changed(\coefs);
	}
	magnitudeResponse { arg viewWidth, minFreq=20.0, maxFreq;
		var response, nyq, sd;
		sd = this.sampleDur;
		nyq = sd.reciprocal * 0.5;
		maxFreq = maxFreq ?? { nyq };
		maxFreq = maxFreq.min(nyq);

        response = Array.fill(viewWidth, { arg x;
			var freq, w, w_cos, w2_cos, numerator, denominator;

			freq = x.linexp(0, viewWidth - 1, minFreq, maxFreq);

			w = 2pi * freq * sd;
			w_cos = w.cos;
			w2_cos = (2*w).cos;
			numerator = (a0*a0)+(a1*a1)+(a2*a2)+(2*((a0*a1)+(a1*a2))*w_cos)
			  +(2*a0*a2*w2_cos);
			denominator = 1.0+(b1*b1)+(b2*b2)+(2*((b1*b2)-b1)*w_cos)
			  -(2*b2*w2_cos);

			(numerator / denominator).pow(1/2).ampdb;
        });

		^response
	}
	phaseResponse  { arg viewWidth, minFreq=20.0, maxFreq;
		var response, nyq, sd;
		sd = this.sampleDur;
		nyq = sd.reciprocal * 0.5;
		maxFreq = maxFreq ?? { nyq };
		maxFreq = maxFreq.min(nyq);

        response = Array.fill(viewWidth, { arg x;
			var freq;

			freq = x.linexp(0, viewWidth, minFreq, maxFreq);

			0.0;
        });

		^response
	}
	/*
	printOn { arg stream;
		stream
			<< this.class.asCompileString << "(" << this.frequency << ", " << this.q << ", " << this.gain << ", " << this.filterKey.asCompileString << ", " << this.sampleRate << ")"
	}
*/
}
