// blackrain 2018
// MagnitudeResponseView a Filter Magnitude Response View

MagnitudeResponseView : SCViewHolder {
    classvar
	<>defaultSelectedCursorColor,
	<>defaultSelectedCursorFillColor,
	<>defaultNonSelectedCursorColor,
	<>defaultNonSelectedCursorFillColor,
	<>defaultDisabledSelectedCursorColor,
	<>defaultDisabledSelectedCursorFillColor,
	<>defaultDisabledNonSelectedCursorColor,
	<>defaultDisabledNonSelectedCursorFillColor,
	<>defaultInfoStringColor,
	<>defaultInfoStringDisabledColor,
	<>defaultinfoStringFont,
	<>defaultFreqMarkersFont,
	<>defaultLevelMarkersFont,
	<>defaultMinFreq = 20.0,
	<>defaultMaxFreq = 20e3,
	<>defaultMaxdb = 12.0,
	<>defaultMaxQ = 50.0,
	<>defaultMinQ = 0.1,
	<>defaultMagnitudeResponseLineColor,
    <>defaultMagnitudeResponseFillColor,
	<>defaultMagnitudeResponseDisabledLineColor,
    <>defaultMagnitudeResponseDisabledFillColor,
	<>defaultPhaseResponseLineColor;

	var editing, <kernels, <selectedKernelIx, <magnitudes, <disabled_magnitudes, <minFreq, <maxFreq,
    <maxdb, <minQ, <maxQ, <>editable=true, <>drawCursors=true, <>freqFine=0.01, <>gainFine=0.01,
    <cursorRect, <>qFine=0.01, <levelMarkers, <freqMarkers, <>cursorWidth=10, <>cursorHeight=10,
    <>freqMarkersFont, <>levelMarkersFont, <>infoStringFont, <>infoStringColor,
    <>infoStringDisabledColor, <>drawInfoString=true, <>drawLevelMarkers=true,
    <>drawFreqMarkers=true, markersImage, <>selectedCursorColor, <>selectedCursorFillColor,
    <>nonSelectedCursorColor, <>nonSelectedCursorFillColor, <>disabledSelectedCursorColor,
    <>disabledSelectedCursorFillColor, <>disabledNonSelectedCursorColor,
    <>disabledNonSelectedCursorFillColor, <>cursorShape=\oval, <>cursorLineWidth=1.5,
    <>fillMagnitudeResponseCurve=true, <>magnitudeResponseLineWidth=1.5,
    <>magnitudeResponseLineColor, <>magnitudeResponseFillColor,
    <>magnitudeResponseDisabledLineColor, <>magnitudeResponseDisabledFillColor,
	<>canSelectFilterKey=true, <>drawPhaseResponse=false, <>phaseResponseLineWidth=1.0,
	<>phaseResponseLineColor, <defaultMouseUpAction, <defaultMouseDownAction,
    <defaultMouseMoveAction;

    *initClass {
        StartUp.add {
			defaultSelectedCursorColor = Color.red;
			defaultSelectedCursorFillColor = Color.blue(0.1,0.1);
			defaultNonSelectedCursorColor = Color.blue(0.7,0.7);
			defaultNonSelectedCursorFillColor = Color.blue(0.1,0.1);
			defaultDisabledSelectedCursorColor = Color.grey;
			defaultDisabledSelectedCursorFillColor = Color.grey;
			defaultDisabledNonSelectedCursorColor = Color.grey(0.5,0.5);
			defaultDisabledNonSelectedCursorFillColor = Color.grey(0.5,0.5);
			defaultInfoStringColor=Color.white;
			defaultInfoStringDisabledColor=Color(0.8,0.8,0.8,1);
			defaultinfoStringFont   = Font("Arial", 11);
			defaultFreqMarkersFont  = Font("Arial", 11);
			defaultLevelMarkersFont = Font("Arial", 11);
            defaultMagnitudeResponseLineColor=Color.cyan;
            defaultMagnitudeResponseFillColor=Color.blue(0.4,0.2);
            defaultMagnitudeResponseDisabledLineColor=Color.grey;
            defaultMagnitudeResponseDisabledFillColor=Color.gray(0.4,0.2);
			defaultPhaseResponseLineColor=Color.black;
        };
    }
	*new { arg parent, bounds;
		^super.new.init(parent, bounds);
	}
	init { arg parent, bounds;
		var selected_kernel, modifier_shift, modifier_ctrl, modifier_alt_ctrl;
		var adjustingQ=false, initialFreq, initialGain, initialQ=1.0, mouseHit,
		fineEdit=false, renderMarkersImage, selectFilterMenuFunc, mouseHitOffset;

		bounds = bounds ? Rect(0,0,400,200);
		this.view_(UserView.new(parent, bounds));

		minFreq = defaultMinFreq;
		maxFreq = defaultMaxFreq;
		maxdb = defaultMaxdb;
		maxQ = defaultMaxQ;
		minQ = defaultMinQ;

		// cursor colors
		selectedCursorColor=defaultSelectedCursorColor;
		selectedCursorFillColor=defaultSelectedCursorFillColor;
		nonSelectedCursorColor=defaultNonSelectedCursorColor;
		nonSelectedCursorFillColor=defaultNonSelectedCursorFillColor;
		disabledSelectedCursorColor=defaultDisabledSelectedCursorColor;
		disabledSelectedCursorFillColor=defaultDisabledSelectedCursorFillColor;
		disabledNonSelectedCursorColor=defaultDisabledNonSelectedCursorColor;
		disabledNonSelectedCursorFillColor=defaultDisabledNonSelectedCursorFillColor;

		// magnitude respose curve colors
        magnitudeResponseLineColor=defaultMagnitudeResponseLineColor;
        magnitudeResponseFillColor=defaultMagnitudeResponseFillColor;
        magnitudeResponseDisabledLineColor=defaultMagnitudeResponseDisabledLineColor;
        magnitudeResponseDisabledFillColor=defaultMagnitudeResponseDisabledFillColor;

		// Phase response curve
		phaseResponseLineColor = defaultPhaseResponseLineColor;

		// markers
		freqMarkersFont = defaultFreqMarkersFont;
		levelMarkersFont = defaultLevelMarkersFont;

		// Info String
		infoStringFont=defaultinfoStringFont;
		infoStringColor=defaultInfoStringColor;
		infoStringDisabledColor=defaultInfoStringDisabledColor;

		editing = false;
        selectedKernelIx = 0;

        freqMarkers = [
			(freq:20, tag:"20Hz"), (freq:30), (freq:40), (freq:50), (freq:60), (freq:70), (freq:80),
			(freq:90), (freq:100, tag:"100Hz"), (freq:200), (freq:300), (freq:400), (freq:500), (freq:600),
			(freq:700), (freq:800), (freq:900), (freq:1000, tag:"1kHz"), (freq:2000), (freq:3000),
			(freq:4000), (freq:5000, /*tag:"5kHz"*/), (freq:6000), (freq:7000), (freq:8000), (freq:9000),
			(freq:10e3, tag:"10kHz"),
			/*	(freq:11e3), (freq:12e3), (freq:13e3), (freq:14e3), (freq:15e3), (freq:16e3), (freq:17e3),
				(freq:18e3), (freq:19e3), */
			(freq:20e3) ];

		levelMarkers = [1, 3, 6, 12, 18, 24, 30, 48, 60, 80, 100];

		this.renderBackgroundImage;

		selectFilterMenuFunc = {
			var menu, kernel, strFilterClasses, classFilterKeys, funcFilterKeys, filterKeys,
			addMenuActions;

			addMenuActions = { arg kernel, menu, keys;
				var keysStr=keys.asArray.sort;
				keysStr.do { arg filterKeyStr;
					var filterKey=filterKeyStr.asSymbol;
					if (kernel.class.filters.at(filterKey).isKindOf(Function)) {
						filterKeyStr = "Func:" + filterKeyStr;
					};
					menu.addAction(
						MenuAction(filterKeyStr,   {
							kernel.selectFilterType(filterKey);
							this.refresh;
							this.action.value(this, kernel, \filterKey, 0, 0);
						})
					);
				};
			};

			if (kernels.size>0) {
				kernel = this.kernels.at(selectedKernelIx);
				if (kernel.enabled==true) {

					filterKeys = kernel.class.filters.keys;

					classFilterKeys = filterKeys.select({|key|
						kernel.class.filters.at(key).className.isKindOf(Class)
					});

					funcFilterKeys = filterKeys.select({|key|
						kernel.class.filters.at(key).className.isKindOf(Function)
					});

					menu = Menu.new;
					addMenuActions.value(kernel, menu, classFilterKeys);
					addMenuActions.value(kernel, menu, funcFilterKeys);
					menu.front;
				};
			};
		};

		this.view.onResize_({ arg view;
			var viewWidth, viewHeight;
			//"onResize".postln;
			viewWidth  = view.bounds.width;
			viewHeight = view.bounds.height;
			if (markersImage.width!=viewWidth or:{markersImage.height != viewHeight }) {
				//renderMarkersImage.value;
				this.renderBackgroundImage;
			};
			view.refresh;
		});

		this.view.drawFunc_({ arg view;
			var scaleZero, viewWidth, viewHeight, info, cursorX, cursorY, kernel, font, stringBounds, stringY,
			xstart, viewHasEnabledKernels, viewHasDisabledKernels, stringColor, phaseResponse;

			viewWidth  = view.bounds.width;
			viewHeight = view.bounds.height;
			scaleZero = (viewHeight * 0.5);

			// guard from double draw
			// method call order is drawFunc -> onResize
			if (viewWidth==markersImage.width or:{viewHeight==markersImage.height}) {
				//"drawFunc".postln;

				markersImage.drawAtPoint(0@0);

				magnitudes  = Array.fill(viewWidth, 0.0);
				disabled_magnitudes = Array.fill(viewWidth, 0.0);

				viewHasEnabledKernels=false;
				viewHasDisabledKernels=false;

				// TODO: speed up- cache magnitudes
				kernels.do {|kernel|
					if (kernel.enabled) {
						viewHasEnabledKernels=true;
						magnitudes = magnitudes + kernel.magnitudeResponse(viewWidth, minFreq, maxFreq);
					}{
						viewHasDisabledKernels=true;
						disabled_magnitudes = disabled_magnitudes + kernel.magnitudeResponse(viewWidth, minFreq, maxFreq);
					};
				};

				// draw disabled filterkernels
				if (viewHasDisabledKernels) {
					if (disabled_magnitudes.size > 0) {
						Pen.width = magnitudeResponseLineWidth;
						Pen.color = magnitudeResponseDisabledLineColor;
						Pen.fillColor = magnitudeResponseDisabledFillColor;

						xstart = disabled_magnitudes.at(0).linlin(maxdb.neg, maxdb, viewHeight, 0);
						Pen.moveTo(Point(0,xstart));

						disabled_magnitudes.do({|y, x|
							y =  y.linlin(maxdb.neg, maxdb, viewHeight, 0);
							Pen.lineTo(x@y);
						});

						if (fillMagnitudeResponseCurve) {
							Pen.lineTo((viewWidth+2)@(disabled_magnitudes.last.linlin(maxdb.neg, maxdb, viewHeight, 0)-2));
							Pen.lineTo((viewWidth+2)@(viewHeight+2));
							Pen.lineTo(-2@(viewHeight+2));
							Pen.lineTo(-2@xstart);

							Pen.fillStroke;
						}{
							Pen.stroke;
						};
					};
				};

				if (viewHasEnabledKernels) {
					Pen.width = magnitudeResponseLineWidth;
					Pen.color = magnitudeResponseLineColor;
					Pen.fillColor = magnitudeResponseFillColor;

					xstart = magnitudes.at(0).linlin(maxdb.neg, maxdb, viewHeight, 0);
					Pen.moveTo(Point(0,xstart));

					magnitudes.do({|y, x|
						y =  y.linlin(maxdb.neg, maxdb, viewHeight, 0);
						Pen.lineTo(x@y);
					});

					if (fillMagnitudeResponseCurve) {
						Pen.lineTo((viewWidth+2)@(magnitudes.last.linlin(maxdb.neg, maxdb, viewHeight, 0)-2));
						Pen.lineTo((viewWidth+2)@(viewHeight+2));
						Pen.lineTo(-2@(viewHeight+2));
						Pen.lineTo(-2@xstart);

						Pen.fillStroke;
					}{
						Pen.stroke;
					};
				};

				if (drawPhaseResponse==true) {
					phaseResponse = Array.fill(viewWidth, 0.0);
					kernels.do({ arg kernel;
						phaseResponse = phaseResponse ++ kernel.phaseResponse(viewWidth, minFreq, maxFreq);
					});

					Pen.width = phaseResponseLineWidth;
					Pen.color = phaseResponseLineColor;

					Pen.moveTo(Point(0,scaleZero));

					phaseResponse.do({|y, x|
						y =  y.linlin(maxdb.neg, maxdb, viewHeight, 0);
						Pen.lineTo(x@y);
					});

					Pen.stroke;
				};

				if (drawCursors==true) {
					kernels.do {|kernel,n|
						var cursorShapeDrawCommand, style, freq, gain;

						// [1]
						freq = kernel.frequency ? 10.0;

						cursorX = freq.explin(minFreq, maxFreq, 0, viewWidth-1.0);

						gain = kernel.gain;
						if (gain.notNil) {
							cursorY = gain.linlin(maxdb.neg, maxdb, viewHeight-1.0, 0);
						}{
							cursorY = scaleZero;
						};

						cursorRect[n] = Rect(
							cursorX-(cursorWidth  * 0.5),
							cursorY-(cursorHeight * 0.5),
							cursorWidth, cursorHeight);

						Pen.width = cursorLineWidth;
                        if (selectedKernelIx == n) {
                            if (kernel.enabled == true) {
                                Pen.strokeColor = selectedCursorColor;
								Pen.fillColor = selectedCursorFillColor;
                            }{
                                Pen.strokeColor = disabledSelectedCursorColor;
								Pen.fillColor = disabledSelectedCursorFillColor;
                            };
                        }{
                            if (kernel.enabled == true) {
                                Pen.strokeColor = nonSelectedCursorColor;
								Pen.fillColor = nonSelectedCursorFillColor;
                            }{
                                Pen.strokeColor = disabledNonSelectedCursorColor;
								Pen.fillColor = disabledNonSelectedCursorFillColor;
                            };
                        };

						case
						{cursorShape==\rect} { cursorShapeDrawCommand=\addRect; style=2; }
						{cursorShape==\fillRect} { cursorShapeDrawCommand=\addRect; style=4;  }
						{cursorShape==\oval} { cursorShapeDrawCommand=\addOval; style=2;  }
						{cursorShape==\fillOval} { cursorShapeDrawCommand=\addOval; style=4;  }
						{ cursorShapeDrawCommand=\addOval; style=4;  };

						Pen.perform(cursorShapeDrawCommand, cursorRect[n]);
						Pen.draw(style);
					};
				};

				if (kernels.notNil and:{ drawInfoString==true }) {

					kernel = kernels.at(selectedKernelIx);
					if (kernel.notNil) {

						info = format("[%] %", selectedKernelIx, kernel.infoString);

						font = infoStringFont;
						stringBounds = info.bounds(font);

						if (kernel.enabled) {
							stringColor=infoStringColor;
						}{
							stringColor=infoStringDisabledColor;
						};

						Pen.use {
							Pen.width_(0.5);
							Pen.color=Color.white;
							Color.black.alpha_(0.4).setFill;
							stringY=view.bounds.height-stringBounds.height-6;
							Pen.fillRect(Rect(2, stringY,
								stringBounds.width+8, stringBounds.height+4));
							Pen.strokeRect(Rect(2, stringY,
								stringBounds.width+8, stringBounds.height+4));
							info.drawAtPoint(Point(4,stringY+2), font, stringColor);
						};
					};
				};
			};
		});

        // modifiers
		//   shift    131072
		//   ctrl     262144
		//   alt+ctrl 786432
		modifier_shift    = 131072;
        modifier_ctrl     = 262144;
        modifier_alt_ctrl = 786432;

		defaultMouseUpAction = { arg view, x, y, modifiers, buttonNumber;
            editing  = false;
            fineEdit = false;
        };

		defaultMouseDownAction = { arg view, x, y, modifiers, buttonNumber, clickCount;
            var kernel;

			if (kernels.size>0) {
				if ( buttonNumber == 0 ) {
					block {|break|
						cursorRect.do {|cursorRect, ix|
							kernel = kernels.at(ix);
							if (cursorRect.contains(Point(x,y)) == true) {
								selectedKernelIx = ix;
								mouseHit = Point(x,y);
								mouseHitOffset = Point(
									x-cursorRect.left-(cursorWidth*0.5)+1.0,
									y-cursorRect.top-(cursorHeight*0.5)+1.0);
								adjustingQ = ((modifiers & modifier_ctrl) != 0);
								fineEdit = ((modifiers & modifier_shift) != 0);
								editing = editable;

								// [2]
								initialFreq = kernel.frequency ? 10.0;
								initialGain = kernel.gain ? 0.0;
								initialQ    = kernel.q ? 1.0;
								view.refresh;
								break.value(ix);
							};
						};
						editing = false;
					};
					if (editing==true) {
						this.action.value(this, kernel, \coefs, modifiers, clickCount);
					};
				}{
					if ( buttonNumber == 1 and:{ this.canSelectFilterKey == true }) {
						selectFilterMenuFunc.value;
					};
				};
			};
        };

		defaultMouseMoveAction = { arg view, x, y, modifiers;
            var freq, gain, newQ, editGain, kernel, qFineInc;
			if (kernels.notNil) {
				kernel = kernels.at(selectedKernelIx);
				if (editing == true and:{ kernel.enabled }) {
					x = x - mouseHitOffset.x + 1.0;
					y = y - mouseHitOffset.y + 1.0;
					if (adjustingQ.not) {
						editGain = kernel.gainKey.notNil;
						if (fineEdit.not) {
							y = y.min(view.bounds.height-1);
							y = y.max(0);
							x = x.min(view.bounds.width-1);
							x = x.max(0);
							freq = x.linexp(0, view.bounds.width-1, minFreq, maxFreq);
							if (editGain) {
								gain = y.linlin(0, view.bounds.height-1, maxdb, maxdb.neg);
							};
						}{
							freq = (initialFreq + ((x - mouseHit.x) * freqFine))
                                   .clip(minFreq,maxFreq);
							if (editGain) {
								gain = (initialGain + ((mouseHit.y - y) * gainFine))
                                       .clip2(maxdb);
							};
						};

						// [3]
						kernel.frequency = freq;
						kernel.gain = gain;
						kernel.paramsChanged;
						view.refresh;
					}{
						if (fineEdit) {
							qFineInc = qFine * 0.1;
						}{
							qFineInc = qFine;
						};
						newQ = (initialQ + ((mouseHit.y - y) * qFineInc)).clip(minQ,maxQ);

						// [4]
						kernel.q = newQ;
						kernel.paramsChanged;
						view.refresh;
					};
					this.action.value(this, kernel, \coefs, modifiers, 0);
				};
			};
        };

		view.mouseUpAction   = defaultMouseUpAction;
        view.mouseDownAction = defaultMouseDownAction;
		view.mouseMoveAction = defaultMouseMoveAction;

		view.mouseWheelAction = { arg view, x, y, modifiers, xDelta, yDelta;
			var numDegrees, numSteps, kernel;
			//[view, x, y, modifiers, xDelta, yDelta].postln;
			numDegrees = yDelta / 8;
			kernel = kernels.at(selectedKernelIx);
			numSteps = numDegrees / 15;
            if (kernel.qKey.notNil) {
                kernel.q = kernel.q + (numSteps * 0.01);
                kernel.paramsChanged;
                view.refresh;
            };
			this.action.value(this, kernel, \coefs, modifiers, 0);
		};

		// keycodes
		// left  65361
		// right 65363
		// up    65362
		// down  65364
		view.keyDownAction_({ arg view, char, modifiers, unicode, keycode;
			var kernel, gainInc, freqInc, qInc, curQ, curGain, curFreq;

			//[view, char, modifiers, unicode, keycode].postln;

			if (this.kernels.size>0) {

				gainInc=0.1;
				freqInc=1.0;
				kernel=this.selectedKernel;

				fineEdit = ((modifiers & modifier_shift) != 0);
				adjustingQ = ((modifiers & modifier_ctrl) != 0);

				if (kernel.enabled) {
					case
					{keycode == 65361} { // left
						curFreq = kernel.frequency;
						if (curFreq.notNil) {
							if (fineEdit) { freqInc = freqInc * 0.001; };
							kernel.frequency_(curFreq - freqInc);
							kernel.paramsChanged;
							this.refresh;
							this.action.value(this, kernel, \coefs, modifiers, 0);
						};
					}
					{keycode == 65363} { // right

						curFreq = kernel.frequency;
						if (curFreq.notNil) {
							if (fineEdit) { freqInc = freqInc * 0.001; };
							kernel.frequency_(curFreq + freqInc);
							kernel.paramsChanged;
							this.refresh;
							this.action.value(this, kernel, \coefs, modifiers, 0);
						};
					}
					{keycode == 65362} { // up
						if (adjustingQ) {
							curQ = kernel.q;
							if (curQ.notNil) {
								if (fineEdit) {
									qInc = qFine * 0.1;
								}{
									qInc = qFine;
								};
								kernel.q_(curQ + qInc);
								kernel.paramsChanged;
								this.refresh;
								this.action.value(this, kernel, \coefs, modifiers, 0);
							};
						}{
							curGain = kernel.gain;
							if (curGain.notNil) {
								if (fineEdit) { gainInc = gainInc * 0.1; };
								kernel.gain_(curGain + gainInc);
								kernel.paramsChanged;
								this.refresh;
								this.action.value(this, kernel, \coefs, modifiers, 0);
							};
						};
					}
					{keycode == 65364} { // down
						if (adjustingQ) {
							curQ = kernel.q;
							if (curQ.notNil) {
								if (fineEdit) {
									qInc = qFine * 0.1;
								}{
									qInc = qFine;
								};
								kernel.q_(curQ - qInc);
								kernel.paramsChanged;
								this.refresh;
								this.action.value(this, kernel, \coefs, modifiers, 0);
							};
						}{
							curGain = kernel.gain;
							if (curGain.notNil) {
								if (fineEdit) { gainInc = gainInc * 0.1; };
								kernel.gain = curGain - gainInc;
								kernel.paramsChanged;
								this.refresh;
								this.action.value(this, kernel, \coefs, modifiers, 0);
							};
						};
					}
					{char == $m} { // Popup Filter Menu
						if (this.canSelectFilterKey == true) {
							selectFilterMenuFunc.value;
						};
					}
					{char == $z} { // zero gain
						kernel.gain_( 0.0 );
						kernel.paramsChanged;
						this.refresh;
						this.action.value(this, kernel, \coefs, modifiers, 0);
					};
				};

				case
				{char == $b} { // bypass/disable
					kernel.enabled_(kernel.enabled.not);
					this.refresh;
					this.action.value(this, kernel, \bypass, modifiers, 0);
				}
				{char == $d} { // bypass/disable
					kernel.enabled_(kernel.enabled.not);
					this.refresh;
					this.action.value(this, kernel, \bypass, modifiers, 0);
				}
				{char == $n} { // next kernel
					if (kernels.size>0) {
						selectedKernelIx=(selectedKernelIx+1).mod(kernels.size);
						this.refresh;
					};
				}
				{char == $p} { // prev. kernel
					if (kernels.size>0) {
						selectedKernelIx=(selectedKernelIx-1).mod(kernels.size);
						this.refresh;
					};
				}
				{char == $-} { // gain zoom -
					this.maxdb_( (maxdb + 1).min(100) );
				}
				{char == $+} { // gain zoom +
					this.maxdb_( (maxdb - 1).max(1) );
				};
			};
		});
    }
	renderBackgroundImage {
		markersImage=Image(this.view.bounds.width.asInteger, this.view.bounds.height.asInteger);
		markersImage.draw({|image|
			var scaleZero, viewWidth, viewHeight, info, cursorX, cursorY, kernel, font,
            stringBounds;

			viewWidth  = image.width;
			viewHeight = image.height;
			scaleZero = (image.height-1.0) * 0.5;

			Pen.smoothing_(false); // antialias off

			Pen.strokeColor = Color.black;
			Pen.color = Color.black;
			Pen.width = 0.5;

			Pen.strokeRect(Rect(0,0,viewWidth-1,viewHeight-1));

			Pen.line(Point(0,scaleZero), Point(viewWidth-1.0,scaleZero));

			if (drawFreqMarkers == true) {
				freqMarkers.do { arg marker;
					var x, freq, tag;
					freq = marker.freq;
					if (freq >=minFreq) {
						x = freq.explin(minFreq, maxFreq, 0, viewWidth);
						Pen.line(Point(x,0), Point(x,viewHeight));
						tag = marker.tag;
						if (tag.notNil) {
							Pen.stringAtPoint(tag, Point(x+2,4), freqMarkersFont);
						};
					};
				};
			};

			if (drawLevelMarkers == true) {
				levelMarkers.do {|marker|
					var y;
					if (marker<maxdb) {
						if (marker.abs>3 or:{maxdb<=7}) { // FIX ME constraint is wrong
							y  = marker.linlin(maxdb.neg, maxdb, 0, viewHeight);
							Pen.line(Point(0,y), Point(viewWidth,y));
							Pen.stringAtPoint(format("% dB",marker.neg), Point(2,y), levelMarkersFont, Color.black);
							y  = marker.neg.linlin(maxdb.neg, maxdb, 0, viewHeight);
							Pen.line(Point(0,y), Point(viewWidth,y));
							Pen.stringAtPoint(format("% dB",marker), Point(2,y), levelMarkersFont, Color.black);
						};
					};
				};
			};

			if (drawFreqMarkers==true or:{ drawLevelMarkers == true }) {
				Pen.stroke;
			};
		});
	}
	levelMarkers_ { arg inMarkers;
		levelMarkers = inMarkers;
		this.renderBackgroundImage; // rebuild back
		this.refresh;
		this.changed(\levelMarkers);
	}
	freqMarkers_ { arg inMarkers;
		freqMarkers = inMarkers;
		this.renderBackgroundImage; // rebuild back
		this.refresh;
		this.changed(\freqMakers);
	}
	minFreq_ { arg inFreq;
		if (minFreq != inFreq) {
			minFreq = inFreq;
			this.renderBackgroundImage; // rebuild back
			this.refresh;
			this.changed(\minFreq);
		};
	}
	maxFreq_ { arg inFreq;
		if (maxFreq != inFreq) {
			maxFreq = inFreq;
			this.renderBackgroundImage; // rebuild back
			this.refresh;
			this.changed(\maxFreq);
		};
	}
	maxdb_  { arg indb;
		if (maxdb != indb) {
			maxdb = indb;
			this.renderBackgroundImage; // rebuild back
			this.refresh;
			this.changed(\maxdb);
		};
	}
	maxQ_  { arg inQ;
		if (maxQ != inQ) {
			maxQ = inQ;
			this.refresh;
			this.changed(\maxQ);
		};
	}
	minQ_  { arg inQ;
		if (minQ != inQ) {
			minQ = inQ;
			this.refresh;
			this.changed(\minQ);
		};
	}
	kernel_ { arg newKernel;
		this.kernels_( newKernel );
	}
    kernels_ { arg newKernels;
        kernels = newKernels.asArray;
		selectedKernelIx = 0;
        cursorRect = Array.fill(kernels.size, { Rect(0,0,0,0); });
        this.refresh;
        this.changed(\kernels);
    }
	selectedKernelIx_ { arg inSelectedKernelIx;
		if (selectedKernelIx != inSelectedKernelIx) {
			selectedKernelIx = inSelectedKernelIx;
			this.refresh;
			this.changed(\selectedKernelIx);
		};
	}
	selectedKernel {
		^this.kernels.at(selectedKernelIx)
	}
}
